/* $Id: eedecode.c,v 1.1 1996/03/26 22:52:36 james Exp $ */
/*
 * $Log: eedecode.c,v $
 * Revision 1.1  1996/03/26 22:52:36  james
 * Initial revision
 *
 */

#include <stdio.h>		/* snprintf() */

#include "jy_sg.h"

/* Config byte values (for the SG Basic 16) */
#define CF_CDROM_TYPE_BIT (1<<7)
#define CF_CDROM_ADDR_BIT (1<<6)
#define CF_GAME_PORT_ENABLED_BIT (1<<5)
#define CF_DSP_ADDR_SELECT_BIT (1<<4)
#define CF_DMA_SELECTION_FIELD ( (1<<3) | (1<<2) )
#define CF_IRQ_SELECTION_FIELD ( (1<<1) | (1<<0) )



/*
 * Decode the config byte for the Sound Galaxy NX II, the SG NX Pro, and 
 * the SG NX Pro 16.
 */

int
decode_sgnx_config_byte(BYTE val, struct sgnx_eeprom_config *p)
{
  p->cd_is_atbus = (val & CF_CDROM_TYPE_BIT);
  p->game_port_enabled = (val & CF_GAME_PORT_ENABLED_BIT);

  p->cdrom_addr  = (val & CF_CDROM_ADDR_BIT) ? 0x300 : 0x320;
  p->dsp_and_mixer_addr = (val & CF_DSP_ADDR_SELECT_BIT) ? 0x220 : 0x240;
  
  switch( (val & CF_DMA_SELECTION_FIELD) >> 2 )
    {
    case 0:
      p->dma = 0; break;
    case 1:
      p->dma = 1; break;
    case 2:
      p->dma = 3; break;		/* NOTE: DMA != 2 HERE. */
    case 3:
      p->dma = -1;		/* invalid! */
      return 0;			/* FAILURE */
    }
  
  switch(val & CF_IRQ_SELECTION_FIELD)
    {
    case 0: p->irq= 2; break;
    case 1: p->irq= 5; break;
    case 2: p->irq= 7; break;
    case 3: p->irq=10; break;
    }
  return 1;			/* success */
}

int
encode_sgnx_config_byte(const struct sgnx_eeprom_config *p, BYTE *pb)
{
  BYTE val;
  BYTE dma_field;
  BYTE irq_field;
  
  val = 0;
  
  if (p->cd_is_atbus)		/* else SCSI */
    val |= CF_CDROM_TYPE_BIT;

  if (p->game_port_enabled)	/* else Disabled */
    val |= CF_GAME_PORT_ENABLED_BIT;
  
  if (0x300 == p->cdrom_addr)	/* else 0x320 */
    val |= CF_CDROM_ADDR_BIT;
  else if (0x320 != p->cdrom_addr)
    return 0;			/* failure */

  if (0x220 == p->dsp_and_mixer_addr) /* else 0x240 */
    val |= CF_DSP_ADDR_SELECT_BIT;
  else if (0x240 != p->dsp_and_mixer_addr)
    return 0;			/* failure */

  switch (p->dma)
    {
    case 0:
      dma_field = 0 << 2;
      break;
    case 1:
      dma_field = 1 << 2;
      break;
    case 3:
      dma_field = 2 << 2;
      break;
    default:
      return 0;			/* failure */
    }
  val |= dma_field;

  switch (p->irq)
    {
    case 2:
      irq_field = 0;
      break;
    case 5:
      irq_field = 1;
      break;
    case 7:
      irq_field = 2;
      break;
    case 10:
      irq_field = 3;
      break;
    default:
      return 0;			/* failure */
    }
  val |= irq_field;

  *pb = val;
  return 1;			/* success */
}

const char *
decode_sgnx_config(const struct sgnx_eeprom_config *pcfg)
{
  static char buf[100];
  
  snprintf(buf, sizeof(buf),
	   "%s CD=0x%03X,Game Port %sabled,DSP=0x%03X,DMA%d,IRQ%d",
	   pcfg->cd_is_atbus ? "AT Bus" : "SCSI",
	   pcfg->cdrom_addr,
	   pcfg->game_port_enabled ? "En" : "Dis",
	   pcfg->dsp_and_mixer_addr,
	   pcfg->dma,
	   pcfg->irq);
  return buf;
}

